/*
 * xlog - GTK+ logging program for amateur radio operators
 * Copyright (C) 2001-2005 Joop Stakenborg <pg4i@amsat.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * gui_utils.c - utilities for the user interface
 */

#include <gtk/gtk.h>
#include "types.h"
#include "log.h"
#include "support.h"

extern GdkColormap *colormap;
extern GtkUIManager *ui_manager;
extern GList *logwindowlist;

/* set style of a label */
void
setlabelstyle (GtkWidget * label, gchar * color, gchar * font)
{
	GdkColor themecolor;
	PangoFontDescription *fnt;

	if (color)
	{
		gdk_color_parse (color, &themecolor);
		if (!gdk_colormap_alloc_color (colormap, &themecolor, FALSE, TRUE))
			g_warning (_("Allocation of color failed in setlabelstyle"));
		gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &themecolor);
	}
	else
		gtk_widget_modify_fg (label, GTK_STATE_NORMAL, NULL);

	if (font)
	{
		fnt = pango_font_description_from_string(font);
		gtk_widget_modify_font(GTK_WIDGET(label), fnt);
		pango_font_description_free(fnt);
	}
}

/* set style of a frame */
void
setframestyle (GtkWidget * frame, gchar * color, gchar * font)
{
	GdkColor themecolor;
	GtkWidget *label;
	PangoFontDescription *fnt;
	GtkStyle *framestyle;

	gdk_color_parse (color, &themecolor);
	if (!gdk_colormap_alloc_color (colormap, &themecolor, FALSE, TRUE))
		g_warning (_("Allocation of color failed in setframestyle"));

	framestyle = gtk_style_copy (frame->style);		
	framestyle->bg[GTK_STATE_NORMAL] = themecolor;
	gtk_widget_set_style (GTK_WIDGET (frame), framestyle);

	label = gtk_frame_get_label_widget (GTK_FRAME(frame));
	gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &themecolor);
	fnt = pango_font_description_from_string(font);
	gtk_widget_modify_font(GTK_WIDGET(label), fnt);
	pango_font_description_free(fnt);
}

/* get color style of a label */
gchar *
getlabelcolor (GtkWidget * label)
{
	GtkStyle *labelstyle;
	GdkColor labelcolor;
	gchar *col;

	labelstyle = gtk_style_copy (label->style);
	labelcolor = labelstyle->fg[GTK_STATE_NORMAL];

	col = g_strdup_printf("#%02X%02X%02X",	 
	labelcolor.red >> 8, labelcolor.green >> 8, labelcolor.blue >> 8);

	return (col);
}

/* configure "Tabs" menu */
void set_tabs_menu (void)
{
	gchar *menupath;
	GtkWidget *menuitem;
	GList *children, *cur;
	logtype *logw;
	gint i;

	for (i = 1; i < 11; i++)
	{
		menupath = g_strdup_printf ("/MainMenu/TabsMenu/%d", i);
		menuitem = gtk_ui_manager_get_widget (ui_manager, menupath);
		gtk_widget_hide (menuitem);
		g_free (menupath);
	}
	for (i = 0; i < g_list_length(logwindowlist); i++)
	{
		if (i < 10)
		{
			menupath = g_strdup_printf ("/MainMenu/TabsMenu/%d", i + 1);
			menuitem = gtk_ui_manager_get_widget (ui_manager, menupath);
			gtk_widget_show (menuitem);
			children = gtk_container_get_children (GTK_CONTAINER (menuitem));
			cur = children;
			logw = g_list_nth_data (logwindowlist, i);
			while (cur) 
			{
				if (GTK_IS_LABEL (cur->data))
					gtk_label_set_text (GTK_LABEL (cur->data), logw->logname);
				cur = cur->next;
			}
			g_free (menupath);
		}
	}
}

void unselect_logs (void)
{
	gint i;
	logtype *logw;
	GtkTreeIter selected;
	GtkTreeModel *model;
	GtkTreeSelection *sel;

	for (i = 0; i < g_list_length (logwindowlist); i++)
	{
		logw = g_list_nth_data (logwindowlist, i);
		sel = gtk_tree_view_get_selection(GTK_TREE_VIEW (logw->treeview));
		if (gtk_tree_selection_get_selected	(sel, &model, &selected))
			gtk_tree_selection_unselect_all (sel);
	}
}
